local folder, core = ...

local module = core:NewModule("Blizzard", "AceEvent-3.0") --, 
local L = LibStub("AceLocale-3.0"):GetLocale(folder, true)

module.name = L.blizzard
module.desc = L.blizzardDesc
module.barMod = true

local regEvents = {
	"UPDATE_BONUS_ACTIONBAR",
}

function module:OnInitialize()
--~ 	Debug("Duel", "OnInitialize")	
	
	self.db = core.db:RegisterNamespace(module.name, {
		profile = {
			moduleEnabled = true,
			hideBarBackground = true,
			barAction = true,
			barBonusAction = true,
			barMultiBarBottomRight = true,
			barMultiBarBottomLeft = true,
			barMultiBarRight = false,
			barMultiBarLeft = false,

		},
	})

	
	if not self.db.profile.moduleEnabled then
		self:SetEnabledState(false)
	end
end

do
	local pairs = pairs
	function module:OnEnable()

--~ 		for i, event in pairs(regEvents) do 
--~ 			self:RegisterEvent(event)
--~ 		end
		
		if self.db.profile.barBonusAction == true then
			self:RegisterEvent("UPDATE_BONUS_ACTIONBAR")
		end
		
		if self.db.profile.hideBarBackground == true then
			self:HideBackgrounds(0)
		else
			self:HideBackgrounds(1)
		end
		
--~ 		self:HookActionButtons()
	end
end


do 
	local green = "|cff00ff00%s|r"
	local red = "|cffff0000%s|r"

	local LibStub = LibStub
	local OnEnable = module.OnEnable
	function module:OnEnable(...)
		if OnEnable then OnEnable(self, ...) end
		core.options.args.barsGroup.args[module.name].name = green:format("* ")..module.name
		LibStub("AceConfig-3.0"):RegisterOptionsTable(core.title, core.options ) --
		
	end
	local OnDisable = module.OnDisable
	function module:OnDisable(...)
		if OnDisable then OnDisable(self, ...) end
		core.options.args.barsGroup.args[module.name].name = red:format("* ")..module.name
		LibStub("AceConfig-3.0"):RegisterOptionsTable(core.title, core.options ) --
	end
end

local name = function(info) 
	local key = info[#info]
--~ 	return key
	return L[key.."Name"] and L[key.."Name"]:format(module.db.profile[key]) or key
end
local desc = function(info) 
	local key = info[#info]
--~ 	return key
	return L[key.."Desc"] and L[key.."Desc"]:format(module.db.profile[key]) or key
end

do
	local ENABLE = ENABLE
	local ACTIONBAR_LABEL = ACTIONBAR_LABEL
	local SHOW_MULTIBAR1_TEXT = SHOW_MULTIBAR1_TEXT
	local OPTION_TOOLTIP_SHOW_MULTIBAR1 = OPTION_TOOLTIP_SHOW_MULTIBAR1
	local SHOW_MULTIBAR2_TEXT = SHOW_MULTIBAR2_TEXT
	local OPTION_TOOLTIP_SHOW_MULTIBAR2 = OPTION_TOOLTIP_SHOW_MULTIBAR2
	local SHOW_MULTIBAR3_TEXT = SHOW_MULTIBAR3_TEXT
	local OPTION_TOOLTIP_SHOW_MULTIBAR3 = OPTION_TOOLTIP_SHOW_MULTIBAR3
	local SHOW_MULTIBAR4_TEXT = SHOW_MULTIBAR4_TEXT
	local OPTION_TOOLTIP_SHOW_MULTIBAR4 = OPTION_TOOLTIP_SHOW_MULTIBAR4
	
	function module:GetOptions()
		return {
			name = self.name,
			type = "group",
			get = function(info)
				local key = info[#info]
				return self.db.profile[key]
			end,
			set = function(info, v)
				local key = info[#info]
				self.db.profile[key] = v
				self:Disable()
				self:Enable()
			end,
			args = {
				Desc = {
					type = "description",
					name = self.desc,
					order = 1,
				},
				moduleEnabled = {
					type = "toggle",	order	= 10,
					name	= ENABLE,
					desc	= desc,
					set = function(info, v)
						self.db.profile.moduleEnabled = v 
						if v == true then
							self:Enable()
						else
							self:Disable()
						end
					end,
				},
				
				
				hideBarBackground = {
					type = "toggle",	order	= 11,
					name	= name,
					desc	= desc,
				},
				
				barsHeader = {
					name	= ACTIONBAR_LABEL,
					order	= 19,
					type = "header",
				},
				
				barAction = {
					type = "toggle",	order	= 20,
					name	= name,
					desc	= desc,
				},
				barBonusAction = {
					type = "toggle",	order	= 21,
					name	= name,
					desc	= desc,
				},

				barMultiBarBottomLeft = {
					type = "toggle",	order	= 22,
					name	= SHOW_MULTIBAR1_TEXT,
					desc	= OPTION_TOOLTIP_SHOW_MULTIBAR1,
				},
				barMultiBarBottomRight = {
					type = "toggle",	order	= 23,
					name	= SHOW_MULTIBAR2_TEXT,
					desc	= OPTION_TOOLTIP_SHOW_MULTIBAR2,
				},
				
				barMultiBarRight = {
					type = "toggle",	order	= 24,
					name	= SHOW_MULTIBAR3_TEXT,
					desc	= OPTION_TOOLTIP_SHOW_MULTIBAR3,
				},
				barMultiBarLeft = {
					type = "toggle",	order	= 25,
					name	= SHOW_MULTIBAR4_TEXT,
					desc	= OPTION_TOOLTIP_SHOW_MULTIBAR4,
				},
			}
		}
	end
end

------------
local blizBars = {
	"MultiBarBottomRight",
	"MultiBarBottomLeft",
	"MultiBarRight",
	"MultiBarLeft",
--~ 	"BonusAction",
--~ 	"Action",
}

do
	local NUM_ACTIONBAR_BUTTONS = NUM_ACTIONBAR_BUTTONS
	local pairs = pairs
	local _G = _G
	local GetBonusBarOffset = GetBonusBarOffset

	--------------------------------------
	function module:HookActionButtons()		--
	-- Hook action button's OnUpdate.	--
	--------------------------------------
		for _, barName in pairs(blizBars) do 
			if self.db.profile["bar"..barName] == true then
				for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
					core:HookActionButton(_G[barName.."Button"..i], barName, i)
				end
			end
		end
		
		if GetBonusBarOffset() > 0 then
			if self.db.profile["bar".."BonusAction"] == true then
				for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
					core:HookActionButton(_G["BonusAction".."Button"..i], "BonusAction", i)
				end
			end
		else
			if self.db.profile["bar".."Action"] == true then
				for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
					core:HookActionButton(_G["Action".."Button"..i], "Action", i)
				end
			end
		end
	end
end

do
	local OnDisable = module.OnDisable
	function module:OnDisable(...)
		if OnDisable then OnDisable(self, ...) end
		self:UnhookAllBars()
		self:HideBackgrounds(1)
	end
end

do
	local pairs = pairs
	----------------------------------------------------------------------
	function module:UnhookAllBars()											--
	-- Unhook all bars, this also returns action buttons to 100% alpha.	--
	----------------------------------------------------------------------
--~ 		Debug("UnhookAllBars")
	--~ 	if BT4 == true then
	--~ 		for i=1, 10 do 
	--~ 			UnhookBar("BT4Bar"..i)
	--~ 		end
	--~ 		
	--~ 	elseif Dominos == true then
	--~ 		for i=1, 10 do 
	--~ 			UnhookBar("dBar"..i)
	--~ 		end
	--~ 		
	--~ 	else
	--~ 		self:UnhookBar("Bar1")
	--~ 		self:UnhookBar("BonusBar1")
	--~ 		self:UnhookBar("MultiBarBottomLeft")
	--~ 		self:UnhookBar("MultiBarBottomRight")
	--~ 		self:UnhookBar("MultiBarRight")
	--~ 		self:UnhookBar("MultiBarLeft")
			
		for _, barName in pairs(blizBars) do 
			core:UnhookBar(barName)
		end
		core:UnhookBar("BonusAction")
		core:UnhookBar("Action")
	--~ 	end
	end
end

do
	local _G = _G
	--Grab the button's iron below a bonus action button.
	local underButtonName, underButton
	module.underIcon = setmetatable({}, {
		__index = function(t,button)
			local underButtonName = "ActionButton"..button:GetID()
			local underButton = _G[underButtonName]
			t[button] = _G[underButtonName.."Icon"] or underButton:GetNormalTexture()
--~ 			print("underIcon", button, button:GetName(), underButton:GetName())
			return t[button]
		end
	})
	
	module.icon = setmetatable({}, {
		__index = function(t,button)
			local buttonName = button:GetName()
			t[button] = _G[buttonName.."Icon"] or buttonName:GetNormalTexture()
--~ 			print("underIcon", button, button:GetName(), underButton:GetName())
			return t[button]
		end
	})
	
end

do
	local _G = _G
	function module:HideBackgrounds(alpha)--	/run _DA:HideBackgrounds(0)
		for i=1, 2 do 
			_G["BonusActionBarFrameTexture"..i]:SetVertexColor(1,1,1,alpha)
		end
		
		for i=0, 1 do 
			_G["MainMenuBarTexture"..i]:SetVertexColor(1,1,1,alpha)
		end
	end
end

do 
	local GetBonusBarOffset = GetBonusBarOffset
	function module:UpdateActionButton(button, name, icon)
		if name:find("BonusAction") then
			self.underIcon[button]:SetVertexColor(1,1,1, 0, true)
		end
--~ 		if name:find("ActionButton") and GetBonusBarOffset() > 0 then
--~ 			icon:SetVertexColor(1,1,1, 0, true)
--~ 			return true
--~ 		end
	end
end

--~ 	"BonusAction",
--~ 	"Action",
do 
	local GetBonusBarOffset = GetBonusBarOffset
	local NUM_ACTIONBAR_BUTTONS = NUM_ACTIONBAR_BUTTONS
	local _G = _G
	
	local actionBar = "Action"
	local bonusActionBar = "BonusAction"
	
	function module:UPDATE_BONUS_ACTIONBAR(event, ...)
--~ 		core.Debug(event, GetBonusBarOffset())
		if GetBonusBarOffset() > 0 then
			for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
				self.icon[_G[actionBar.."Button"..i]]:SetVertexColor(1,1,1,0,true)
			end
			core:UnhookBar(actionBar)
			
			for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
				core:HookActionButton(_G[bonusActionBar.."Button"..i], bonusActionBar, i)
			end
		else
			core:UnhookBar(bonusActionBar)
			
			for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
				core:HookActionButton(_G[actionBar.."Button"..i], "Action", i)
			end
		end
	end
end